const { promisify } = require('util')
const figlet = promisify(require('figlet'))
const download = promisify(require('download-git-repo'))
const { spawn } = require('child_process')
const ora = require('ora')
const clear = require('clear')
const chalk = require('chalk')
const inquirer = require('inquirer')
const shell = require('shelljs')

const log_blue = (content) => {
  console.log(chalk.blue.bold(content))
}

const log_success = (content) => {
  console.log(chalk.green.bold(content))
}

const log_normal = (content) => {
  console.log(chalk.whiteBright.bold.bgYellow(content))
}

const clone = async (repo, desc) => {
  const process = ora(`🚘 下载 ${repo} 中...`)
  process.start()
  await download(repo, desc)
  process.succeed()
}

const spawnHandle = async (...args) => {
  return new Promise(resolve => {
    const porc = spawn(...args)
    porc.stdout.pipe(process.stdout)
    porc.stderr.pipe(process.stderr)
    porc.on('close', () => {
      resolve()
    })
  })
}

const askQuestion = (prompts) => {                    //询问交互
  return (files, metalsmith, done) => {
      async.eachSeries(Object.keys(prompts), (key, next) => {
          prompt(metalsmith.metadata(), key, prompts[key], next)
      }, done)
  }
}

exports.runInit = async (name) => {
  // 清空终端
  clear()
  const data = await figlet('CETC', {
    font: 'isometric3'
  })
  log_normal(data)
  log_blue('')
  log_blue(`🐷🐷🐷 创建react后台模版到【${name}文件夹】......`)
  await clone('github:Yanzengyong/cetc-react-admin-start', name)
  log_success('😋 模版下载成功！！！')

  const hasLogin = await inquirer.prompt([
    {
      type: 'confirm',
      message: '模版中是否需要login（登陆）页面？',
      name: 'template'
    }
  ])

  if (hasLogin.template) {
    shell.rm('-rf', `./${name}/src/layout/index.js`);
    shell.rm('-rf', `./${name}/src/routes/index.js`);
    shell.cd(`./${name}/src/layout/`)
    shell.mv('./spare.js', './index.js')
    shell.cd(`../`)
    shell.cd(`./routes/`)
    shell.mv('./spare.js', './index.js')
  } else {
    console.log(`./${name}/src/layout/spare.js`)
    shell.rm('-rf', `./${name}/src/layout/spare.js`);
    shell.rm('-rf', `./${name}/src/routes/spare.js`);
    shell.rm('-rf', `./${name}/src/pages/Login`);
  }

  log_blue('开始安装依赖...')
  await spawnHandle('yarn', [], { cwd: `./${name}` })
  log_success(`
    ✨ 安装完成！！！
    ☁️  运行模版：
    🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷
    ====================================================================
      cd ${name}
      ----------------------
      yarn start
    ====================================================================
    🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷 🐷
  `)
}
